// -*- C++ -*-
#include "Rivet/Rivet.hh"
#include "Rivet/Particle.hh"
#include "Rivet/ParticleName.hh"
#include "Rivet/Event.hh"
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/ZFinder.hh"
#include <cmath>

namespace Rivet {
  
  /// @
  class LHCB_2012_I1208102 : public Analysis {
  public:
    
    /// Default constructor
    LHCB_2012_I1208102()
    : Analysis("LHCB_2012_I1208102")
    {    }
    
    
    /// @name Analysis methods
    //@{
    
    /// Book histograms
    void init() {
      FinalState fs;
      ZFinder zeefinder(fs, 2.0, 4.5, 20.*GeV, PID::ELECTRON, 60.*GeV, 120.*GeV,
                        0.1, ZFinder::CLUSTERNODECAY);
      addProjection(zeefinder, "ZeeFinder");
      
      _h_sigma_vs_y = bookHisto1D(2, 1, 1);
      _h_sigma_vs_phi = bookHisto1D(3, 1, 1);
      
    }
    
    
    
    /// Do the analysis
    void analyze(const Event& e) {
      const double weight = e.weight();
      const ZFinder& zeefinder = applyProjection<ZFinder>(e, "ZeeFinder");
      //if (zeefinder.bosons().size() != 1) vetoEvent;
      if (zeefinder.bosons().size() < 1) {
        MSG_WARNING("Could not find Z bosons in current event...");
        return;
      }
      
      if (zeefinder.bosons().size() > 1) {
        MSG_WARNING("Found " << zeefinder.bosons().size() << " decays Z -> e+ e-...");
      }
      // Z momenta
      const FourMomentum& zee = zeefinder.bosons()[0].momentum();
            
      const Particle& pozitron = zeefinder.constituents()[0];
      const Particle& electron = zeefinder.constituents()[1];
      
      //Calculation of the angular variable
      double diffphi = Rivet::deltaPhi(pozitron,electron);
      double diffpsd = Rivet::deltaEta(pozitron,electron);
      double accphi=PI-diffphi;
      double angular=tan(accphi/2)/cosh(diffpsd/2);
      
      _h_sigma_vs_y->fill(zee.rapidity(), weight);
      _h_sigma_vs_phi->fill(angular, weight);
    }
    
    
    /// Finalize
    void finalize() {
      const double xs = crossSection()/picobarn;
      scale(_h_sigma_vs_y, xs/sumOfWeights());
      scale(_h_sigma_vs_phi, xs/sumOfWeights());
    }
    //@}
    
  private:
    
    /// @name Histograms
    //@{
    Histo1DPtr _h_sigma_vs_y;
    Histo1DPtr _h_sigma_vs_phi;
    
    //@}
    
  };
  
  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(LHCB_2012_I1208102);
}
